;; This program is written in a non-conventional style resembling that of
;; C. Later programs will deviate from this style and move more towards
;; the Common Lisp style.

(in-package :iolib.examples)

;; this implements an iterative server which handles one connection and then
;; exits.

(defun run-ex1-server (&key (port *port*))
  ;; Create a passive (server) TCP socket under IPV4
  (let ((socket
         (make-socket 
          :connect :passive
          :address-family :internet
          :type :stream 
          :external-format '(:utf-8 :eol-style :crlf)
          :ipv6 nil)))
       (format t "Created socket: ~A[fd=~A]~%"
               socket (socket-os-fd socket))

       ;; Bind the socket to all interfaces with specified port.
       (bind-address socket 
                     +ipv4-unspecified+ 
                     :port port
                     :reuse-addr t)
       (format t "Bound socket: ~A~%" socket)

       ;; convert the socket to a listening socket
       (listen-on socket :backlog 5)
       (format t "Listening on socket bound to: ~A:~A~%"
               (local-name socket)
               (local-port socket))

       ;; block on accepting connection
       (format t "Waiting to accept a connection...~%")
       (let ((client (accept-connection socket :wait t)))
            ;; When we get a new connection, show who it is from.
            (multiple-value-bind (remote rport) (remote-name client)
                                 (format t "Got a connection from ~A:~A!~%"
                                         remote rport))

            (format t "Sending the time...")
            ;; Since we're using a internet TCP stream, we can use format
            ;; with it. However, we should be sure to finish-output in 
            ;; order that all the data is sent. Also, this is a blocking
            ;; write.
            (multiple-value-bind (s m h d mon y) (get-decoded-time)
                                 (format client "~A/~A/~A ~A:~A:~A~%" 
                                         mon d y h m s) 
                                 (finish-output client))

            ;; we're done talking to the client.
            (close client)
            (format t "Sent!~%"))

       ;; we're done with the server socket too.
       (close socket)
       t))

