;;;; This is a more common-lisp-like style, and to be used for the rest
;;;; of the examples.

(in-package :iolib.examples)

(defun run-ex3-server-helper (port)
  (with-open-socket
   (server :connect :passive
           :address-family :internet
           :type :stream
           :ipv6 nil
           :external-format '(:utf-8 :eol-style :crlf))
   (format t "Created socket: ~A[fd=~A]~%"
           server (socket-os-fd server))

   ;; Bind the socket to all interfaces with specified port.
   (bind-address server +ipv4-unspecified+
                 :port port
                 :reuse-addr t)
   (format t "Bound socket: ~A~%" server)

   ;; start listening on the server socket
   (listen-on server :backlog 5)
   (format t "Listening on socket bound to: ~A:~A~%"
           (local-name server)
           (local-port server))

   ;; keep accepting connections forever.
   (do ()
       (nil nil)
       (format t "Waiting to accept a connection...~%")
       (with-accept-connection (client server :wait t)
                               ;; When we get a new connection, show who it 
                               ;; is from.
                               (multiple-value-bind 
                                (remote rport) (remote-name client)
                                (format t "Got a connnection from ~A:~A!~%"
                                        remote rport))
                               ;; Since we're using a internet TCP stream, 
                               ;; we can use format with it. However, we 
                               ;; should be sure to finish-output in order that 
                               ;; all the data is sent.
                               (multiple-value-bind
                                (s m h d mon y) (get-decoded-time)
                                (format t "Sending the time...")
                                (handler-case
                                 (progn
                                  (format client "~A/~A/~A ~A:~A:~A~%"
                                          mon d y h m s)
                                  (finish-output client))
                                 (hangup
                                  ()
                                  (format t "Client closed conection!~%")))
                                (format t "Sent!~%"))))
   t))

(defun run-ex3-server (&key (port *port*))
  (handler-case
   (run-ex3-server-helper port)

   (socket-address-in-use-error
    ()
    (format t "Bind: Address already in use, forget :reuse-addr t?"))))
