(in-package :iolib.examples)

(defun run-ex4-client-helper (port)

  ;; Create a internet TCP socket under IPV4
  (with-open-socket
   (socket :connect :active
           :address-family :internet
           :type :stream 
           :external-format '(:utf-8 :eol-style :crlf)
           :ipv6 nil)

   ;; do a blocking connect to the daytime server on the port.
   (connect socket (lookup-hostname "localhost") :port port :wait t)

   (format t "Connected to server ~A:~A from my local connection at ~A:~A!~%"
           (remote-name socket) (remote-port socket)
           (local-name socket) (local-port socket))

   (handler-case
   ;; read the one line of information I need form the daytime server.
   ;; I can use read-line here because this is a TCP socket. It will block
   ;; until the whole line is read.

    (let ((line (read-line socket)))
         (format t "~A" line)
         t)

    ;; However, let's notice the signaled condition when the server went 
    ;; away prematurely...
    (end-of-file 
     ()
     (format t "Got end-of-file. Server closed connection!")))))

(defun run-ex4-client (&key (port *port*))
  (handler-case
   (run-ex4-client-helper port)

   ;; handle a commonly signaled error...
   (socket-connection-refused-error 
    ()
    (format t "Connection refused to ~A:~A. Maybe the server isn't running?~%" 
            (lookup-hostname "localhost") port))))
