
(in-package :iolib.examples)

(defvar *client-threads* nil)
(defvar *tls-client* nil)

(defun run-ex4-server-helper (port)
  (with-open-socket
   (server :connect :passive
           :address-family :internet
           :type :stream
           :ipv6 nil
           :external-format '(:utf-8 :eol-style :crlf))
   (format t "Created socket: ~A[fd=~A]~%"
           server (socket-os-fd server))

   ;; Bind the socket to all interfaces with specified port.
   (bind-address server +ipv4-unspecified+
                 :port port
                 :reuse-addr t)
   (format t "Bound socket: ~A~%" server)

   ;; start listening on the server socket
   (listen-on server :backlog 5)
   (format t "Listening on socket bound to: ~A:~A~%"
           (local-name server)
           (local-port server))

   ;; keep accepting connections forever, but if this exits for whatever
   ;; reason ensure to destroy any remaining running threads.
   (unwind-protect
    (do ()
        (nil nil)
        (declare (special *tls-client*))
        (format t "Waiting to accept a connection...~%")
        ;; set up a thread to handle the client.
        (let* ((client (accept-connection server :wait t))
               (*default-special-bindings* 
                `((*tls-client* . ,(funcall (lambda () client))) 
                  ,@*default-special-bindings*)))

              ;; keep track of any active threads we currently know about...
              (setf *client-threads* 
                    (remove-if-not #'thread-alive-p *client-threads*))
              (push 
               ;; ...and handle the connection!
               (make-thread #'process-ex4-client-thread) 
               *client-threads*)))

    ;; clean up all of the client threads when done.
    (format t "Destroying any active threads....~%")
    (mapc #'(lambda (thr) 
                    ;; XXX What do I do about the race between the check and
                    ;; the destroy?
                    (when (thread-alive-p thr) 
                          (format t "Destroying: ~A~%" thr)
                          (destroy-thread thr)))
          *client-threads*)
    (setf *client-threads* nil))))

;;; The thread which handles the client connection.
(defun process-ex4-client-thread ()
  (declare (ignorable *tls-client*))
  (let ((client *tls-client*))
       (multiple-value-bind 
        (remote rport) (remote-name client)
        (format t "A thread is handling the connection from ~A:~A!~%"
                remote rport))
       ;; Since we're using a internet TCP stream, 
       ;; we can use format with it. However, we 
       ;; should be sure to finish-output in order that 
       ;; all the data is sent.
       (multiple-value-bind
        (s m h d mon y) (get-decoded-time)
        (handler-case
         (progn
          (format t "Sending the time...")
          (format client "~A/~A/~A ~A:~A:~A~%"
                  mon d y h m s)
          (finish-output client)
          (close client)
          (format t "Sent!~%")
          t)

         (hangup
          ()
          (format t "Client closed connection.~%")
          (close client)
          t)))))


;; This just checks for some error conditions so we can print out a nice
;; message about it.
(defun run-ex4-server (&key (port *port*))
  (handler-case
   (run-ex4-server-helper port)
   ;; handle some common signals
   (socket-address-in-use-error
    ()
    (format t "Bind: Address already in use, forget :reuse-addr t?"))))


