(in-package :iolib.examples)

(defun run-ex5-client-helper (port)

  ;; Create a internet TCP socket under IPV4
  (with-open-socket
   (socket :connect :active
           :address-family :internet
           :type :stream 
           :external-format '(:utf-8 :eol-style :crlf)
           :ipv6 nil)

   ;; do a blocking connect to the daytime server on the port.
   (connect socket (lookup-hostname "localhost") :port port :wait t)

   (format t "Connected to server ~A:~A from my local connection at ~A:~A!~%"
           (remote-name socket) (remote-port socket)
           (local-name socket) (local-port socket))

   (handler-case
    (str-cli socket)
    (hangup 
     ()
     (format t "Got hangup. Server closed connection on write!"))
    (end-of-file 
     ()
     (format t "Got end-of-file. Server closed connection on read!")))))

;; read a line from stdin, write it to the server, read the response, write
;; it to stdout. If we read 'quit' then echo it to the server and leave the 
;; loop. All of this is blocking i/o.
(defun str-cli (socket)
  (let ((done nil))
       (do ()
           (done t)
           (let ((line (read-line)))
                ;; send it to the server, get the response.
                (format socket "~A~%" line)
                (finish-output socket)
                (format t "~A~%" (read-line socket))

                (when (equal line "quit")
                      (setf done t))))))

(defun run-ex5-client (&key (port *port*))
  (handler-case
   (run-ex5-client-helper port)

   ;; handle a commonly signaled error...
   (socket-connection-refused-error 
    ()
    (format t "Connection refused to ~A:~A. Maybe the server isn't running?~%" 
            (lookup-hostname "localhost") port))))
